#include<avr/io.h>
#include <avr/interrupt.h>
#include <math.h>
#include "inverter.h"


unsigned int PWM_TOP;
volatile unsigned char sample_N;
volatile unsigned char sine_table[50];
volatile unsigned char phase0, phase1, phase2;


void pwm_init(void)
{
	DDRB |= 0xE0;	// OCR1A, OCR1B, OCR1C 
	DDRE |= 0x38;	// OCR3A, OCR3B, OCR3C 
	// PWM mode 14
	// 14.7456MHz
	TCCR1A = 0xBA;	//PWM, Fast PWM
	TCCR1B = 0x19;
	TCCR1C = 0x00;

	TCCR3A = 0xEE;	//PWM, Fast PWM
	TCCR3B = 0x19;
	TCCR3C = 0x00;

	TIMSK |= 0x04;
	TIFR = 0x04;
	sei();
}

void pwm_set(unsigned int pwmFreq,unsigned int pwmDuty)
{
	PWM_TOP = F_CPU/pwmFreq;
	pwmDuty = (unsigned long)pwmFreq*pwmDuty/100;
	// set top count value
	ICR1H = (unsigned char)(PWM_TOP>>8);
	ICR1L = (unsigned char)PWM_TOP;

	OCR1AH = (unsigned char)(pwmDuty>>8);
	OCR1AL = (unsigned char)pwmDuty;

	OCR1BH = (unsigned char)(pwmDuty>>8);
	OCR1BL = (unsigned char)pwmDuty;

	OCR1CH = (unsigned char)(pwmDuty>>8);
	OCR1CL = (unsigned char)pwmDuty;

	// set top count value
	ICR3H = (unsigned char)(PWM_TOP>>8);
	ICR3L = (unsigned char)PWM_TOP;

	OCR3AH = (unsigned char)(pwmDuty>>8);
	OCR3AL = (unsigned char)pwmDuty;

	OCR3BH = (unsigned char)(pwmDuty>>8);
	OCR3BL = (unsigned char)pwmDuty;

	OCR3CH = (unsigned char)(pwmDuty>>8);
	OCR3CL = (unsigned char)pwmDuty;
}

void U_pwm_duty(unsigned int pwmDuty)
{
	pwmDuty = (unsigned long)PWM_TOP*pwmDuty/100;

	OCR1AH = (unsigned char)(pwmDuty>>8);
	OCR1AL = (unsigned char)pwmDuty;

	OCR1BH = (unsigned char)(pwmDuty>>8);
	OCR1BL = (unsigned char)pwmDuty;
}

void V_pwm_duty(unsigned int pwmDuty)
{
	pwmDuty = (unsigned long)PWM_TOP*pwmDuty/100;

	OCR1CH = (unsigned char)(pwmDuty>>8);
	OCR1CL = (unsigned char)pwmDuty;

	OCR3AH = (unsigned char)(pwmDuty>>8);
	OCR3AL = (unsigned char)pwmDuty;
}

void W_pwm_duty(unsigned int pwmDuty)
{
	pwmDuty = (unsigned long)PWM_TOP*pwmDuty/100;

	OCR3BH = (unsigned char)(pwmDuty>>8);
	OCR3BL = (unsigned char)pwmDuty;

	OCR3CH = (unsigned char)(pwmDuty>>8);
	OCR3CL = (unsigned char)pwmDuty;
}

void phase_init(unsigned char sample)
{
    sample_N = sample;
	phase0 = 0;
    phase1 = 2*sample/3;	//-120
    phase2 = 1*sample/3;	//-240
}

void MakeSineTable(unsigned int amplitude){ // smaple_N >= 3;
	unsigned int i;
	float theta;

	//	0 ~ 1/4
	sine_table[0]=amplitude/2 + 1;

	for (i = 1; i < sample_N/4; i++){
		theta = (M_PI_2 * (i / (float)(sample_N/4-1)));
		sine_table[i] = amplitude/2 * (1+sin(theta));
	}

	//	1/4 ~ 2/4
	sine_table[sample_N/4]=amplitude - 1;

	for (i = 1; i < sample_N/4; i++){
		sine_table[sample_N/4 + i] = sine_table[sample_N/4 - i];
	}

	//	2/4 ~ 3/4
	for (i = 0; i < sample_N/4; i++) {
		sine_table[i + sample_N/2] = amplitude - sine_table[i];
	}

	//	3/4 ~ 4/4
	for (i = 0; i < sample_N/4; i++) {
		sine_table[i + sample_N/2 + sample_N/4] = amplitude - sine_table[sample_N/4+i];
	}
}

SIGNAL(SIG_OVERFLOW1)
{
	U_pwm_duty(sine_table[phase0]);
	V_pwm_duty(sine_table[phase1]);
	W_pwm_duty(sine_table[phase2]);

	if (++phase0 == sample_N) phase0 = 0;
	if (++phase1 == sample_N) phase1 = 0;
	if (++phase2 == sample_N) phase2 = 0;
}
