#include<avr/io.h>
#include <avr/interrupt.h>
#include <compat/deprecated.h>
#include "bldc_upper_pwm.h"


unsigned int PWM_TOP;
volatile unsigned char sample_N;
volatile unsigned char sine_table[20];
volatile unsigned char phase0, phase1, phase2;


void pwm_init(void)
{
	DDRB |= 0xE0;	// OCR1A, OCR1C 
	DDRE |= 0x38;	// OCR3B 
	// PWM mode 14
	// 14.7456MHz
	TCCR1A = 0x8A;	//PWM, Fast PWM
	TCCR1B = 0x19;
	TCCR1C = 0x00;

	TCCR3A = 0x22;	//PWM, Fast PWM
	TCCR3B = 0x19;
	TCCR3C = 0x00;

	sei();
}

void pwm_set(unsigned int pwmFreq,unsigned int pwmDuty)
{
	PWM_TOP = F_CPU/pwmFreq;
	pwmDuty = (unsigned long)pwmFreq*pwmDuty/100;
	// set top count value
	ICR1H = (unsigned char)(PWM_TOP>>8);
	ICR1L = (unsigned char)PWM_TOP;

	OCR1AH = (unsigned char)(pwmDuty>>8);
	OCR1AL = (unsigned char)pwmDuty;

	OCR1CH = (unsigned char)(pwmDuty>>8);
	OCR1CL = (unsigned char)pwmDuty;

	// set top count value
	ICR3H = (unsigned char)(PWM_TOP>>8);
	ICR3L = (unsigned char)PWM_TOP;

	OCR3BH = (unsigned char)(pwmDuty>>8);
	OCR3BL = (unsigned char)pwmDuty;
}

void U0_pwm_duty(unsigned int pwmDuty)
{
	pwmDuty = (unsigned long)PWM_TOP*pwmDuty/100;

	OCR1AH = (unsigned char)(pwmDuty>>8);
	OCR1AL = (unsigned char)pwmDuty;
}

void V0_pwm_duty(unsigned int pwmDuty)
{
	pwmDuty = (unsigned long)PWM_TOP*pwmDuty/100;

	OCR1CH = (unsigned char)(pwmDuty>>8);
	OCR1CL = (unsigned char)pwmDuty;
}

void W0_pwm_duty(unsigned int pwmDuty)
{
	pwmDuty = (unsigned long)PWM_TOP*pwmDuty/100;

	OCR3BH = (unsigned char)(pwmDuty>>8);
	OCR3BL = (unsigned char)pwmDuty;
}

void U0_pwm_On(void)
{
	// turn on channel A (OC1A) PWM output
	// set OC1A as non-inverted PWM
	sbi(TCCR1A,COMA1);
	cbi(TCCR1A,COMA0);
}

void V0_pwm_On(void)
{
	// turn on channel C (OC1C) PWM output
	// set OC1C as non-inverted PWM
	sbi(TCCR1A,COMC1);
	cbi(TCCR1A,COMC0);
}

void W0_pwm_On(void)
{
	// turn on channel B (OC3B) PWM output
	// set OC3B as non-inverted PWM
	sbi(TCCR3A,COMB1);
	cbi(TCCR3A,COMB0);
}

void U0_pwm_Off(void)
{
	// turn off channel A (OC1A) PWM output
	// set OC1A (OutputCompare action) to none
	cbi(TCCR1A,COMA1);
	cbi(TCCR1A,COMA0);
}

void V0_pwm_Off(void)
{
	// turn off channel C (OC1C) PWM output
	// set OC1B (OutputCompare action) to none
	cbi(TCCR1A,COMC1);
	cbi(TCCR1A,COMC0);
}

void W0_pwm_Off(void)
{
	// turn off channel B (OC3B) PWM output
	// set OC3B (OutputCompare action) to none
	cbi(TCCR3A,COMB1);
	cbi(TCCR3A,COMB0);
}

void U1_gpio_On(void)
{
	sbi(PORTB,PB6);
}

void V1_gpio_On(void)
{
	sbi(PORTE,PE3);
}

void W1_gpio_On(void)
{
	sbi(PORTE,PE5);
}

void U1_gpio_Off(void)
{
	cbi(PORTB,PB6);
}

void V1_gpio_Off(void)
{
	cbi(PORTE,PE3);
}

void W1_gpio_Off(void)
{
	cbi(PORTE,PE5);
}

void gpio_init(void)
{
	DDRC = 0x00;	// Rotory switch
	DDRA = 0x00;	// Hall state
}

unsigned char D_Digit(void)
{
	if((PINC&0x0F) > 10) return 10;
	return PINC&0x0F;
}

unsigned char BLDC_Hall_Status(void)
{
	return PINA&0x07;
}

void run_bldc(unsigned char hall_state)
{
	switch(hall_state)
	{
		case state1:
			//U0_On.
			//V1_On.
			U0_pwm_On();	//On
			U1_gpio_Off();

			V0_pwm_Off();
			V1_gpio_On();	//On

			W0_pwm_Off();
			W1_gpio_Off();
		break;
		case state2:
			//U0_On.
			//W1_On.
			U0_pwm_On();	 //On
			U1_gpio_Off();

			V0_pwm_Off();
			V1_gpio_Off();

			W0_pwm_Off();
			W1_gpio_On();	//On
		break;
		case state3:
			//V0_On.
			//W1_On.
			U0_pwm_Off();
			U1_gpio_Off();

			V0_pwm_On();	//On
			V1_gpio_Off();

			W0_pwm_Off();
			W1_gpio_On();	//On
		break;
		case state4:
			//V0_On.
			//U1_On.
			U0_pwm_Off();
			U1_gpio_On();	//On

			V0_pwm_On();	//On
			V1_gpio_Off();

			W0_pwm_Off();
			W1_gpio_Off();
		break;
		case state5:
			//W0_On.
			//U1_On.
			U0_pwm_Off();
			U1_gpio_On();	//On

			V0_pwm_Off();
			V1_gpio_Off();

			W0_pwm_On();	//On
			W1_gpio_Off();
		break;
		case state6:
			//W0_On.
			//V1_On.
			U0_pwm_Off();
			U1_gpio_Off();

			V0_pwm_Off();
			V1_gpio_On();	//On

			W0_pwm_On();	//On
			W1_gpio_Off();
		break;
	}
}
